;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x6b69c960a6f11bbe0feb5c8f9566dfc6 . "./code.cg2js/cg-status-bar.fasl")
(#xc1cf92d79da49c30be187d1a37f81137 . "./code.cg2js/cg-hotspot.fasl")
(#x04ae8a6f39c5307065a02fc7f787d3cc . "./code.cg2js/cg-toolbar.fasl")
(#xff57395cc34f1def402ac30855976785 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x8f6e44558453ccafed3d496d35163094 . "./code.cg2js/cg-tooltip.fasl")
(#xc1e8727e776bd138354d8b9f9adcdccc . "./code.cg2js/cg-calendar.fasl")
(#x0ed71c67529be6316a16920378ea1ea4 . "./code.cg2js/cg-item-list.fasl")
(#xa7d09bcd05eb54ead1dee99dd4e57528 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x740387928e3388ac8a4eb0a9a941ccdd . "./code.cg2js/cg-trackbar.fasl")
(#x079f88fb7fa741a437631b5b6039c8be . "./code.cg2js/cg-class-grid.fasl")
(#xe410d85b1c8c3ff9b404dfee5c204d42 . "./code.cg2js/cg-dialog-item.fasl")
(#x9dcf8f47eb1597cc150c9d14cb5cbc66 . "./code.cg2js/cg-comtab.fasl")
(#xa82c302a23283133d7e94134e6cfb449 . "./code.cg2js/cg-lisp-text.fasl")
(#x543713c01ae3b23f4b555a22c8c00a4a . "./code.cg2js/cg-button.fasl")
(#x8593de4865dcbb9313a74f06c088a582 . "./code.cg2js/cg-printing.fasl")
(#x0b59d7ee1157056574360022a97da6ea . "./code.cg2js/cg-pixmap-printing.fasl")
(#x465ea74b920f8fbe8ab4d00b69b28105 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xc3ea41ebaffd15640d2c3d2f8ffa7a6f . "./code.cg2js/cg-pixmap-widget.fasl")
(#x48be41338e78100a7211dc14a90afc3f . "./code.cg2js/cg-selected-object.fasl")
(#x12e84899357a2a9f551faad58460dd35 . "./code.cg2js/cg-text-or-combo.fasl")
(#x8c0eeb789d7a299fe1de05e727bfe93e . "./code.cg2js/cg-graphics-context.fasl")
(#xc8d4420fdcb31cbe80a4f672956f9ffb . "./code.cg2js/cg-edit-in-place.fasl")
(#xac3db70babacb36db2b2655e64bcf228 . "./code.cg2js/cg-string-dialog.fasl")
(#xf91b29ad734b2aa30941f785fdd0155f . "./code.cg2js/cg-list-view.fasl")
(#x0b58ac1057df2975547f03616c378837 . "./code.cg2js/cg-intersect.fasl")
(#xffcee017d8a159467a7bfba7ef5f6cf1 . "./code.cg2js/cg-text-widget.fasl")
(#xea93eb471aaf30457c068c912b7ad84b . "./code.cg2js/cg-directory-dialog.fasl")
(#x4aeb7a213356c0ba2d5fa11ddebc40dc . "./code.cg2js/cg-file-dialog.fasl")
(#x68d75ae8959de311b13fca0ef050f7e8 . "./code.cg2js/cg-dde-utils.fasl")
(#x4fdaaa76dd887f93798df84991c301ff . "./code.cg2js/cg-nodes.fasl")
(#x1804f75cc37e5db6c516fb10a99c92ed . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x018770fe417024173666a13fd8b46739 . "./code.cg2js/cg-up-down-control.fasl")
(#x928d9149d7e92e29ef56f837bc87e558 . "./code.cg2js/cg-get-position.fasl")
(#x7579c540b414113d706b2b10c2a9084d . "./code.cg2js/cg-toggling-widget.fasl")
(#xdf5711ca230ac12548e23cfe8d6c44ee . "./code.cg2js/cg-acache.fasl")
(#x6c846c5b3a5a0c486972d2fa1c35e5b0 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xc3ca401ade73a5d5b19f6ee7cd5d031b . "./code.cg2js/cg-base.fasl")
(#x53fd555d316f678f8bffa371cf6c8559 . "./code.cg2js/cg-drag-and-drop.fasl")
(#xd6e4313df1636201578e8ff9064a3ce4 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x182c2e5cfc27e9d985c4929bf194d35f . "./code.cg2js/cg-property.fasl")
(#xae5714daa0c1a630297a72655344411f . "./code.cg2js/cg-editable-text.fasl")
(#x81377f7ea9f965aa07385496310ffefb . "./code.cg2js/cg-lamp.fasl")
(#x02d3572141ad6df2702724b7146cd85a . "./code.cg2js/cg-progress-indicator.fasl")
(#xfe4625b5f3e9b49cab12c80897bdf796 . "./code.cg2js/cg-chart-widget.fasl")
(#x1b5f0a3da67083c35ca137f09184fa9f . "./code.cg2js/cg-common-control.fasl")
(#x028d5b5878a18dadc27ccbbd652c95ea . "./code.cg2js/cg-font-dialog.fasl")
(#x0b6a28a0edb39187fb9d8b999c0afc2f . "./code.cg2js/cg-gesture-emulation.fasl")
(#xd9741db247b4220c0c00fa057fdacb06 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x7b49756f0c4202b985a4030f23c8c8d6 . "./code.cg2js/cg-os-window.fasl")
(#x6391df71da25d39e37f7b58e1a1179b3 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x5e33db50dcc8f6ebb8e03cb89df6bfa2 . "./code.cg2js/cg-pixmap.fasl")
(#x6d9fd17da0596fbc515b39b31b62b672 . "./code.cg2js/cg-chart-or-plot.fasl")
(#xbc9bb4b9d79f11fd63e3e963ee5b0757 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xcaa0a07b55dff3ca604855eabc09033f . "./code.cg2js/cg-scaling-stream.fasl")
(#xc49ea6dbaae14297bec1ac67d1e0741a . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x7c986e55b6bbd5718aa8d4e46c8106c9 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x041f24e3789cfbee73145286d1f69d6e . "./code.cg2js/cg-timer.fasl")
(#x98eb73d4a59740480f7d32905b07138e . "./code.cg2js/cg-scroll-bar.fasl")
(#xf11c64ab2a92e3f1eaec6473aa8ff576 . "./code.cg2js/cg-caret.fasl")
(#xbf51b269dc91fda2bc490a13681d1c2c . "./code.cg2js/cg-template-string.fasl")
(#x6dedcf076f39cca5396598e83d9fda78 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xf0fa6ec7933dec1b3821c4e9237f4d8c . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x8a78751d6208b7136e71fb3644f2bf81 . "./code.cg2js/cg-combo-box.fasl")
(#x3045380b9da9bfebdbeb7c90181522f2 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x520b781948dea44d3590f126b9316e44 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x55aff82fdb178a12027fad579a0ac5f8 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x79e387c97d55178b2aa9fdd14dd1ce97 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xdff81d5c72e8c5f6a056bb9249e07a9a . "./code.cg2js/cg-player.fasl")
(#xd959461c4884fa6256bc7e3982656515 . "./code.cg2js/cg-outline.fasl")
(#x7cce14909b8b8c2f71ef702cb47ed0e6 . "./code.cg2js/cg-class-support.fasl")
(#x57bf28fe30a1f7de3e990ba5d3090461 . "./code.cg2js/cg-lettered-menu.fasl")
(#xfa00baf67822743ac77a1f45157cb59d . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xb9db3afb7f37d424de561ec9588afa66 . "./code.cg2js/cg-fill-texture.fasl")
(#x28d782ac9368549b6ef6b76b4c76a5fe . "./code.cg2js/cg-lisp-widget.fasl")
(#x350b23dea6571ea1e245aa746d9f2a34 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xba269ffdc34a1279e8263fccdc49ff93 . "./code.cg2js/cg-clipboard.fasl")
(#x35c31f8b092b5af4728195a8f95d429a . "./code.cg2js/cg-wrap-string.fasl")
(#x2540016b6df522d05bfa9ee3ea32963b . "./code.cg2js/cg-os-widget.fasl")
(#x5ed75d539eff561b308e37da7ed65bff . "./code.cg2js/cg-check-box.fasl")
(#xc0117421a6fa4953edccb1356b9b0c74 . "./code.cg2js/cg-choice-list.fasl")
(#x12d1729da310ec2feb244e128e76d675 . "./code.cg2js/cg-layout.fasl")
(#xd649e553db3adf1e0a6e4acce5a72b64 . "./code.cg2js/cg-group-box.fasl")
(#x15a5bec2307212c32df8cda7a3d9352a . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x979356019d13c7837a3dc1cbaf265ba5 . "./code.cg2js/cg-radio-button.fasl")
(#x8e4447dceebecd0954a4a2b7affe5889 . "./code.cg2js/cg-tab-control.fasl")
(#x493e6247cb096899848b2cc16248f2c4 . "./code.cg2js/cg.fasl")
(#x23bf7064d20e7175472251f2053773a5 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x0c942cb885e61c670a575163a02d5b04 . "./code.cg2js/cg-dropping-outline.fasl")
(#xe163f90e08095c2b7b5f84696204503e . "./code.cg2js/cg-get-pixmap.fasl")
(#xa1fa6e75ce31179bb1340ce0a3e24bd9 . "./code.cg2js/cg-grid-widget.fasl")
(#x5db486a740e575051f7ad8e96984e720 . "./code.cg2js/cg-utility-dialog.fasl")
(#x9d85728d79823ab046ff913201714a83 . "./code.cg2js/cg-static-text.fasl")
(#xd1ec3118a2122c484b9bc608758a169c . "./code.cg2js/cg-class-slot-grid.fasl")
(#x8a0ad5c7161bf1416a3fd51c600b565d . "./code.cg2js/cg-object-editor-layout.fasl")
(#x64f58129ccbbba050b81cf2a37ad6fff . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x98efb34b735deb5b585b0fcb9e468735 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x73d83b577930d0bf1e0674d1dfcb92e4 . "./code.cg2js/cg-plot-widget.fasl")
(#x53cfa752636c6a9a7ab5492a1b0b761a . "./code.cg2js/cg-color-dialog.fasl")
(#x198c73c8c27cc8bd69aaf298ae64f6b3 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x375ee78f8a8a3a2671d168c18865a989 . "./code.cg2js/cg-icon.fasl")
(#x3e4da54968221b6036a634b0a0bb7331 . "./code.cg2js/cg-dev.fasl")
(#x366bdd3056b8943715de75943f504f1f . "./code.cg2js/cg-choose-printer.fasl")
(#x4e048471da1c0e4cf20e2fc4a704665d . "./code.cg2js/cg-intersect-posbox.fasl")
(#x15304c52adeb3f8f8456de1610a34032 . "./code.cg2js/cg-menu.fasl")
(#x8ecf1be514e1bf1c6146e3ee0ad347f9 . "./code.cg2js/cg-yes-no-list.fasl")
(#xbe62c55f6d7ab2225b490feb6068a26a . "./code.cg2js/cg-palette.fasl")
(#xc555dc167f547569e665b082cb048e38 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x69552b1ee4ab82618c59947506d144a8 . "./code.cg2js/cg-message-dialog.fasl")
(#x3f3adc9f69ca593e9fbbff7cb05b8ff8 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xaaade73a3b8a281c1fd5c620493f804f . "./code.cg2js/cg-split-bar.fasl")
(#x3dd33d277bc3fc29f69096946a5b1a45 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xe2dc53b0ba09bb7fc5aefd90172f7e78 . "./code.cg2js/cg-web-browser.fasl")
(#xc5c5ccdb33924ef43dbd675f05d3bc4e . "./code.cg2js/cg-project-window.fasl")
(#x8f13b63ed60d16d9efbb98a4f8532f90 . "./code.cg2js/cg-picture-widget.fasl")
(#x7c81ea8646cd7d47a757b03a58f0942f . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x73002b4b0568bfb2b274d8cc0734de31 . "./code.cg2js/cg-object-editor.fasl")
(#xfeb7327221f59777c1c1981292ca2260 . "./code.cg2js/cg-paren-matching.fasl")
(#x1087dc581805a062dc99528378bc18cd . "./code.cg2js/cg-sample-file-menu.fasl")
(#xb43d3767e75940794e2822f45c25547e . "./code.cg2js/cg-drawable.fasl")
(#xe6b08aec58a7808200f028866c1f1a16 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xee06d79b12d042719c0d30a0d5107c37 . "./code.cg2js/cg-html-widget.fasl")
(#x167528f77edede913cbb20f1ed53da64 . "./code.cg2js/cg-curve.fasl")
(#x0c9d3e71582c6fdaa4e7c9421a2866f6 . "./code.cg2js/cg-yes-no-string.fasl")
(#x5e02f8e5007b756ea50ca28f28604ed9 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xb833edeb256ebda8a9880d9163563775 . "./code.cg2js/cg-html-dialog.fasl")
(#x770fe4a0b86b4d6324e5d34be4c5959d . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x1530d081108f718e323bd9a9d1b518d4 . "./code.cg2js/cg-tray.fasl")
(#x9bd716329e97b550815c97aa6c0572ea . "./code.idejs/ide-apropos.fasl")
(#xda472c02c0d0e5b115edcdae28aa65c1 . "./code.idejs/ide-window-list.fasl")
(#x01b1bf4723b5c70a471708d97dde46ae . "./code.idejs/ide-class-browser.fasl")
(#xfda46f9d5de12f32c1b162a69ef8718f . "./code.idejs/ide-base.fasl")
(#x895bbf8ce27926782a77a32233b4463f . "./code.idejs/ide-process-browser.fasl")
(#x274f0374933b43f742771fe4ae7533fc . "./code.idejs/ide-acache.fasl")
(#xa844b5d3f1688551b4ccadcd0366cd14 . "./code.idejs/ide-navigator.fasl")
(#x880397989df915cc3d980c822c87cf79 . "./code.idejs/ide-trace.fasl")
(#x3a2925fa4753fd02ed08430cc3a42053 . "./code.idejs/ide-project.fasl")
(#x2b7a0fb326b475f172cbec5ea84726d8 . "./code.idejs/ide-patch-dialog.fasl")
(#xf9fac0012d4ab80819f255f6a9fbf107 . "./code.idejs/ide-grapher.fasl")
(#x41c89443bffe69a706d241829ae8ec6f . "./code.idejs/ide-dev.fasl")
(#x3d2822594e5038839230b2fd078d28aa . "./code.idejs/ide-keyshort.fasl")
(#x058109eabf0fedce132d7a9afea6179c . "./code.idejs/ide-editor.fasl")
(#x1b7201787cd0e1671536d4e64e4077bf . "./code.idejs/ide-inspector.fasl")
(#xd50c64f426126d9670f02cf67c71d46e . "./code.idejs/ide.fasl")
(#x9a1f379956d0ee00b8e4ae687ab05a84 . "./code.idejs/ide-clipboard.fasl")
(#xbcfa11bea67419f581ec7535a56a4754 . "./code.idejs/ide-find-definitions.fasl")
(#x488073cbce995a775215cfc7a23ca030 . "./code.idejs/ide-form.fasl")
(#x726aad5a626db935629e670b649e0190 . "./code.idejs/ide-intro.fasl")
(#x23589072eaa33450c52eee9cba25a7d2 . "./code.idejs/ide-find-in-files.fasl")
(#x5baf69cfbcb0e01a09e5f2c6a9ab5978 . "./code.idejs/ide-package-list.fasl")
(#x8e8849d1ebc828c6192ce80b39b9d37f . "./code.idejs/ide-class-grapher.fasl")
(#x9a8f50ffdba20a10156ef75920aa924b . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (format t "Incorrect checksum on file: ~a" (cdr checksum))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
